import re

from django.db.models import Q
from .models import Name


def normalize_query(query_string):
    """Splits the query string in individual keywords, removing
    unnecessary spaces and grouping quoted words together.

    Example:
    >>> normalize_query('some random words "with quotes" and spaces')
        ['some', 'random', 'words', 'with quotes', 'and', 'spaces']
    """
    findterms = re.compile(r'"([^"]+)"|(\S+)').findall
    normspace = re.compile(r'\s{2,}').sub

    terms = (findterms(query_string))

    return [normspace(' ', (t[0] or t[1]).strip()) for t in terms]


def compose_query(query_string):
    """Composes a Q object from the query_string.

    This will first normalize the query_sting and split the string up
    into words. Then it will iterate over the list of words creating Q
    object for each one. Last, it reduces the generated Q objects into
    a single Q object by combining them using the & operator.

    The reduce function essential performs the following operation for
    a query_string that contains 4 words,

        (((Q1 & Q2) & Q3) & Q4)

    where Q1-4 are instances of the Q object.
    """
    # Normalize the query_string which will also split the single string
    # into a list of terms.
    terms = normalize_query(query_string)

    # Create a generator that produces an instance of the Q object
    # for each term.
    qs = (Q(name__icontains=term) for term in terms)

    # Use bitwise AND to compose the objects into a single instance of Q.
    return reduce(lambda x, y: x & y, qs)


def resolve_type(q_type):
    """Resolve the Name Types passed to the request, and returns
    a list of Name Type IDs.

    The input is expected to be the raw q_type query string value passed
    in with the request.  The input is expected to be a comma delimited
    list such as, `Personal,Event,Software`.  This function will lookup
    up the ID of the Name Type based on the string and return a list
    of Name Type IDs.
    """
    if not q_type:
        return []

    q_type = q_type.split(',')

    # Iterate through the Name Types and create a list of Name IDs
    # using the corresponding strings values in q_type
    return [k for k, v in Name.NAME_TYPE_CHOICES if v in q_type]


# TODO: Look at reducing the number of queries.
def filter_names(request):
    """Return the set of Name objects filtered on the query
    parameters passed with the request.
    """
    q = request.GET.get('q', None)
    name_types = resolve_type(request.GET.get('q_type', None))

    # Retrieve the visible names from the database.
    names = Name.objects.visible()

    # Filter by name type if it is passed with the request.
    if name_types:
        names = names.filter(name_type__in=name_types)

    # Do further filtering if the q parameter is present.
    if q:
        # All names that fit query OR all variants that contain query
        query_filter = compose_query(q) | Q(variant__variant__icontains=q)
        names = names.filter(query_filter).distinct()

    return names



import json
from datetime import datetime
from itertools import groupby

import markdown2
from django.core.urlresolvers import reverse
from django.db import models, transaction
from django.utils import timezone
from django.utils.six.moves.urllib.parse import quote
from django.utils.six.moves.urllib.request import urlopen
from pynaco.naco import normalizeSimplified

from .validators import validate_merged_with


class Identifier_Type(models.Model):
    """Custom Identifier Type.

    Used in conjunction with the Identifier model.
    """
    label = models.CharField(
        max_length=255,
        help_text='What kind of data is this? Personal website? Twitter?')

    icon_path = models.CharField(
        max_length=255,
        blank=True,
        help_text='Path to icon image?')

    homepage = models.URLField(
        blank=True,
        help_text='Homepage of label. Twitter.com, Facebook.com, etc')

    class Meta:
        ordering = ['label']
        verbose_name = 'Identifier Type'

    def __unicode__(self):
        return self.label


class Identifier(models.Model):
    """An Identifier for a Name models instance. Most commonly
    represented as a permalink.

    This is used in conjunction with the Identifier Type model to
    specify the type of Identifier the instance represents. An example
    instance would have an Identifier Type of `Twitter` and the value
    field would have the permalink to the Name's Twitter profile page.
    """
    type = models.ForeignKey(
        'Identifier_Type',
        help_text="Catagorize this record's identifiers here")

    belong_to_name = models.ForeignKey('Name')
    value = models.CharField(max_length=500)
    visible = models.BooleanField(default=True)
    order = models.IntegerField(default=0)

    class Meta:
        ordering = ['order', 'type']

    def __unicode__(self):
        return self.value


class NoteManager(models.Manager):
    """Custom Model Manager for the Note model."""
    use_for_related_fields = True

    def public_notes(self):
        return self.get_queryset().exclude(note_type=self.model.NONPUBLIC)


class Note(models.Model):
    """A note regarding the related Name model instance."""
    BIOGRAPHICAL_HISTORICAL = 0
    DELETION_INFORMATION = 1
    NONPUBLIC = 2
    SOURCE = 3
    OTHER = 4

    NOTE_TYPE_CHOICES = (
        (BIOGRAPHICAL_HISTORICAL, 'Biographical/Historical'),
        (DELETION_INFORMATION, 'Deletion Information'),
        (NONPUBLIC, 'Nonpublic'),
        (SOURCE, 'Source'),
        (OTHER, 'Other')
    )

    note = models.TextField(help_text='Enter notes about this record here')
    note_type = models.IntegerField(choices=NOTE_TYPE_CHOICES)
    belong_to_name = models.ForeignKey('Name')

    objects = NoteManager()

    def get_note_type_label(self):
        """Returns the label associated with an instance's
        note_type.
        """
        id, note_type = self.NOTE_TYPE_CHOICES[self.note_type]
        return note_type

    def __unicode__(self):
        return self.note


class Variant(models.Model):
    """Defines an alternative form that a Name may be displayed."""
    ACRONYM = 0
    ABBREVIATION = 1
    TRANSLATION = 2
    EXPANSION = 3
    OTHER = 4

    VARIANT_TYPE_CHOICES = (
        (ACRONYM, 'Acronym'),
        (ABBREVIATION, 'Abbreviation'),
        (TRANSLATION, 'Translation'),
        (EXPANSION, 'Expansion'),
        (OTHER, 'Other')
    )

    belong_to_name = models.ForeignKey('Name')
    variant_type = models.IntegerField(
        choices=VARIANT_TYPE_CHOICES,
        help_text='Choose variant type.')

    variant = models.CharField(
        max_length=255,
        help_text='Fill in the other name variants, if any.')

    normalized_variant = models.CharField(
        max_length=255,
        editable=False,
        help_text='NACO normalized variant text')

    def get_variant_type_label(self):
        """Returns the label associated with an instance's
        variant_type.
        """
        id, variant_type = self.VARIANT_TYPE_CHOICES[self.variant_type]
        return variant_type

    def save(self):
        self.normalized_variant = normalizeSimplified(self.variant)
        super(Variant, self).save()

    def __unicode__(self):
        return self.variant


class TicketingManager(models.Manager):
    """Custom manager for the BaseTicketing model."""

    def create(self, *args, **kwargs):
        """Override the create method.

        Create or get the single object. If one already exists,
        delete it and create a new one so we auto-increment the id.
        """
        obj, created = self.get_or_create(stub=self.model.STUB_DEFAULT)
        if not created:
            with transaction.atomic():
                obj.delete()
                obj = self.create(stub=self.model.STUB_DEFAULT)
        return obj


class BaseTicketing(models.Model):
    """Creates a custom app-level identifier.

    This leverages the autoincrement primary key field to
    create custom unique identifier. An example identifier
    would be `nm0000001`.
    """
    # Explicitly set the id of the model, even though it is the same
    # as the one Django gives it.
    id = models.AutoField(null=False, primary_key=True)

    # This is just the smallest placeholder we can create that we can
    # replace into to generate a new id.
    STUB_DEFAULT = True
    stub = models.BooleanField(null=False, default=STUB_DEFAULT, unique=True)

    # Override the default manager
    objects = TicketingManager()

    @property
    def ticket(self):
        """Alias for id"""
        return self.id

    def __unicode__(self):
        return u'nm{ticket:07d}'.format(ticket=self.ticket)


class NameManager(models.Manager):
    """Custom Manager for the Name model.

    Provides additional methods that are useful in calculating
    statistics on Name model instances.
    """

    def visible(self):
        """Retrieves all Name objects that have an Active record status
        and are not merged with any other Name objects.
        """
        return self.get_queryset().filter(
            record_status=self.model.ACTIVE, merged_with=None)

    def active_type_counts(self):
        """Calculates counts of Name objects by Name Type.

        Statistics are based off of the queryset returned by visible.
        The total number is calculated using the count method. All
        additional figures are calculated using Python to reduce the number
        of queries.
        """
        names = self.visible()
        return {
            'total': names.count(),
            'personal': len(filter(lambda n: n.is_personal(), names)),
            'organization': len(filter(lambda n: n.is_organization(), names)),
            'event': len(filter(lambda n: n.is_event(), names)),
            'software': len(filter(lambda n: n.is_software(), names)),
            'building': len(filter(lambda n: n.is_building(), names))
        }

    def _counts_per_month(self, date_column):
        """Calculates the number of Names by month according to the
        date_column passed in.

        This will return a ValueQuerySet where each element is in the form
        of
            {
               count: <Number of Names for the month>,
               month: <Datetime object for first day of the given month>
            }
        """
        def grouper(name):
            return (getattr(name, date_column).year,
                    getattr(name, date_column).month)

        def convert_key(year, month):
            datetime_obj = datetime(year=year, month=month, day=1)
            tzinfo = timezone.get_current_timezone()
            return timezone.make_aware(datetime_obj, tzinfo)

        results = self.all().order_by(date_column)

        return [
            dict(month=convert_key(*key), count=len(list(value)))
            for key, value in groupby(results, grouper)
        ]

    def created_stats(self):
        """Returns a list of the number of Names created per
        month.
        """
        return self._counts_per_month('date_created')

    def modified_stats(self):
        """Returns a list of the number of Names modified per
        month.
        """
        return self._counts_per_month('last_modified')


class Name(models.Model):
    """The authorized version of a name that is used to unambiguously
    refer to a person, organization, event, building or piece of
    software.
    """
    ACTIVE = 0
    DELETED = 1
    SUPPRESSED = 2

    RECORD_STATUS_CHOICES = (
        (ACTIVE, 'Active'),
        (DELETED, 'Deleted'),
        (SUPPRESSED, 'Suppressed')
    )

    PERSONAL = 0
    ORGANIZATION = 1
    EVENT = 2
    SOFTWARE = 3
    BUILDING = 4

    NAME_TYPE_CHOICES = (
        (PERSONAL, 'Personal'),
        (ORGANIZATION, 'Organization'),
        (EVENT, 'Event'),
        (SOFTWARE, 'Software'),
        (BUILDING, 'Building')
    )

    DATE_DISPLAY_LABELS = {
        PERSONAL: {
            'type': 'Personal',
            'begin': 'Date of Birth',
            'end': 'Date of Death'
        },
        ORGANIZATION: {
            'type': 'Organization',
            'begin': 'Founded Date',
            'end': 'Defunct'
        },
        EVENT: {
            'type': 'Event',
            'begin': 'Begin Date',
            'end': 'End Date'
        },
        SOFTWARE: {
            'type': 'Software',
            'begin': 'Begin Date',
            'end': 'End Date'
        },
        BUILDING: {
            'type': 'Building',
            'begin': 'Erected Date',
            'end': 'Demolished Date',
        },
        None: {
            'type': None,
            'begin': 'Born/Founded Date',
            'end': 'Died/Defunct Date'
        }
    }

    NAME_TYPE_SCHEMAS = {
        PERSONAL: 'http://schema.org/Person',
        ORGANIZATION: 'http://schema.org/Organization',
        BUILDING: 'http://schema.org/Place'
    }

    name = models.CharField(
        max_length=255,
        help_text='Please use the general reverse order: LAST, FIRST')

    normalized_name = models.CharField(
        max_length=255,
        editable=False,
        help_text='NACO normalized form of the name')

    name_type = models.IntegerField(choices=NAME_TYPE_CHOICES)

    # Date, month or year of birth or incorporation of the name
    begin = models.CharField(
        max_length=25,
        blank=True,
        help_text='Conforms to EDTF format YYYY-MM-DD')

    # Date, month of year of death or un-incorporation of the name
    end = models.CharField(
        max_length=25,
        blank=True,
        help_text='Conforms to EDTF format YYYY-MM-DD')

    disambiguation = models.CharField(
        max_length=255,
        blank=True,
        help_text='Clarify to whom or what this record pertains.')

    biography = models.TextField(
        blank=True,
        help_text='Compatible with MARKDOWN')

    record_status = models.IntegerField(
        default=ACTIVE,
        choices=RECORD_STATUS_CHOICES)

    merged_with = models.ForeignKey(
        'self',
        blank=True,
        null=True,
        related_name='merged_with_name')

    date_created = models.DateTimeField(auto_now_add=True, editable=False)
    last_modified = models.DateTimeField(auto_now=True, editable=False)
    name_id = models.CharField(max_length=10, unique=True, editable=False)

    objects = NameManager()

    def get_absolute_url(self):
        """Get the absolute url to the Name detail page."""
        return reverse('name:detail', args=[self.name_id])

    def get_schema_url(self):
        """Get the appropriate schema url based on the name type."""
        return self.NAME_TYPE_SCHEMAS.get(self.name_type, None)

    def get_name_type_label(self):
        """Get the string form of the Name's name type."""
        id, name_type = self.NAME_TYPE_CHOICES[self.name_type]
        return name_type

    def get_date_display(self):
        """Get the date display labels according to the Name's
        name type

        See Name.DATE_DISPLAY_LABELS
        """
        return self.DATE_DISPLAY_LABELS.get(self.name_type)

    def has_current_location(self):
        """True if the Name has a current location in the location_set."""
        return self.location_set.current_location is not None

    def has_geocode(self):
        """True if the instance has one or more related Locations."""
        if self.location_set.count():
            return True
        else:
            return False
    has_geocode.boolean = True  # Enables icon display in the Django admin.

    def has_schema_url(self):
        """True if the instance has a schema url."""
        return self.get_schema_url() is not None

    def _is_name_type(self, type_id):
        """Test if the instance of Name is a certain
        Name Type.

        Accepts the id of the Name Type, and returns a boolean.
        """
        return type_id == self.name_type

    def is_personal(self):
        """True if the Name has the Name Type Personal."""
        return self._is_name_type(self.PERSONAL)

    def is_organization(self):
        """True if the Name has the Name Type Organization."""
        return self._is_name_type(self.ORGANIZATION)

    def is_event(self):
        """True if the Name has the Name Type Event."""
        return self._is_name_type(self.EVENT)

    def is_software(self):
        """True if the Name has the Name Type Software."""
        return self._is_name_type(self.SOFTWARE)

    def is_building(self):
        """True if the Name has the Name Type Building."""
        return self._is_name_type(self.BUILDING)

    def _is_record_status(self, status_id):
        """Test if the instance of Name has a particular
        record_status.

        Accepts the id of the Name Type, and returns a boolean.
        """
        return status_id == self.record_status

    def is_active(self):
        """True if the Name has the Active status."""
        return self._is_record_status(self.ACTIVE)

    def is_deleted(self):
        """True if the Name has the Deleted status."""
        return self._is_record_status(self.DELETED)

    def is_suppressed(self):
        """True if the Name has the Suppressed status."""
        return self._is_record_status(self.SUPPRESSED)

    def render_biography(self):
        """Render the Markdown biography to HTML."""
        return markdown2.markdown(self.biography)

    def __normalize_name(self):
        """Normalize the name attribute and assign it the normalized_name
        attribute.
        """
        self.normalized_name = normalizeSimplified(self.name)

    def __find_location(self):
        """Use the normalized_name attribute and the Location.URL to
        attempt to find the instance's location.

        A location is only attached if the server responds with a single
        result.
        """
        URL_RESOURCE = 'http://maps.googleapis.com/maps/api/geocode/json'
        URL_QUERY_TEMPLATE = '?address={address}&sensor=true'
        URL = URL_RESOURCE + URL_QUERY_TEMPLATE

        url = URL.format(address=quote(self.normalized_name))
        payload = json.load(urlopen(url))

        # Only add the location if the Name matched one and only one
        # location from the API.
        if payload.get('status') == "OK" and len(payload.get('results')) == 1:
            coordinate = payload['results'][0]['geometry']['location']
            self.location_set.create(latitude=coordinate['lat'],
                                     longitude=coordinate['lng'])

    def __assign_name_id(self):
        """Use the BaseTicketing object to assign a name_id."""
        if not self.name_id:
            self.name_id = unicode(BaseTicketing.objects.create())

    def save(self, **kwargs):
        self.__normalize_name()
        self.__assign_name_id()
        super(Name, self).save()
        if self.is_building() and not self.location_set.count():
            self.__find_location()

    def clean(self, *args, **kwargs):
        # Call merged_with_validator here so that we can pass in
        # the model instance.
        validate_merged_with(self)
        super(Name, self).clean(*args, **kwargs)

    def __unicode__(self):
        return self.name_id

    class Meta:
        ordering = ['name']
        unique_together = (('name', 'name_id'),)


class LocationManager(models.Manager):
    """Custom Manager for the Location model."""
    use_for_related_fields = True

    def _get_current_location(self):
        """Filters through a Name object's related locations and
        returns the one marked as current.
        """
        return self.get_queryset().filter(status=self.model.CURRENT).first()

    # Makes the current location available as a property on
    # the RelatedManager.
    current_location = property(_get_current_location)


class Location(models.Model):
    """Defines the location of a related Name model instance."""
    CURRENT = 0
    FORMER = 1

    LOCATION_STATUS_CHOICES = (
        (CURRENT, 'current'),
        (FORMER, 'former')
    )

    HELP_TEXT = """
    <strong>
        <a target="_blank" href="http://itouchmap.com/latlong.html">
            iTouchMap
        </a>
        : this service might be useful for filling in the lat/long data
    </strong>
    """

    belong_to_name = models.ForeignKey('Name')

    latitude = models.DecimalField(
        max_digits=13,
        decimal_places=10,
        help_text=HELP_TEXT)

    longitude = models.DecimalField(
        max_digits=13,
        decimal_places=10,
        help_text=HELP_TEXT)

    status = models.IntegerField(
        choices=LOCATION_STATUS_CHOICES,
        default=CURRENT)

    objects = LocationManager()

    class Meta:
        ordering = ['status']

    def geo_point(self):
        return '{lat} {lng}'.format(lat=self.latitude, lng=self.longitude)

    def is_current(self):
        """True if the Location has a status of Current."""
        return self.CURRENT == self.status

    def save(self, **kwargs):
        super(Location, self).save()
        # When this instance's status is CURRENT, get all other locations
        # related the belong_to_name, and set the status to FORMER.
        if self.is_current():
            former_locs = self.belong_to_name.location_set.exclude(id=self.id)
            for l in former_locs:
                l.status = self.FORMER
                l.save()

    def __unicode__(self):
        return self.geo_point()



def jsonp(f):
    """Wrap a json response in a callback, and set the mimetype (Content-Type)
    header accordinglly (will wrap in text/javascript if there is a callback).
    If the "callback" or "jsonp" parameters are provided, will wrap the json
    output in callback({thejson})

    Usage:

    @jsonp
    def my_json_view(request):
        d = { 'key': 'value' }
        return HTTPResponse(json.dumps(d), content_type='application/json')

    """
    from functools import wraps

    @wraps(f)
    def jsonp_wrapper(request, *args, **kwargs):
        resp = f(request, *args, **kwargs)
        if resp.status_code != 200:
            return resp
        if 'callback' in request.GET:
            callback = request.GET['callback']
            resp['Content-Type'] = 'application/javascript; charset=utf-8'
            resp.content = "%s(%s)" % (callback, resp.content)
            return resp
        else:
            return resp

    return jsonp_wrapper



"""Serializers for the Name App Models.

This module leverages the Django Rest Framework's Serializer
components to build JSON representations of the models defined
in this app.

These JSON representations are designed to be backwards compatible
with the API documented in previous versions.

For documentation regarding the Django Rest Framework Serializers go
to http://www.django-rest-framework.org/api-guide/serializers/
"""

from rest_framework import serializers
from .. import models


class IdentifierSerializer(serializers.ModelSerializer):
    """Serializer for the Identifier Model.

    The following fields have been renamed for backwards compatibility
    with previous versions of the API.
        label -> identifier.type
        href -> identifier.value
    """
    label = serializers.StringRelatedField(source='type')
    href = serializers.CharField(source='value')

    class Meta:
        model = models.Identifier
        fields = ('label', 'href')


class NoteSerializer(serializers.ModelSerializer):
    """Serializer for the Note Model."""
    type = serializers.SerializerMethodField()

    class Meta:
        model = models.Note
        fields = ('note', 'type')

    def get_type(self, obj):
        """Sets the type field.

        Returns the Note Type label, instead of the Note Type ID, which
        is the default behavior.
        """
        return obj.get_note_type_label().lower()


class VariantSerializer(serializers.ModelSerializer):
    """Serializer for the Variant Model."""
    type = serializers.SerializerMethodField()

    class Meta:
        model = models.Variant
        fields = ('variant', 'type')

    def get_type(self, obj):
        """Sets the type field.

        Returns the Variant Type label, instead of the Variant Type ID,
        which is the default behavior.
        """
        return obj.get_variant_type_label().lower()


class NameSerializer(serializers.ModelSerializer):
    """Serializer for the Name Model.

    This serializes the the Name model to include detailed information
    about the object, including the related Variants, Notes, and
    Identifiers.

    The following fields have been renamed for backwards compatibility
    with previous versions of the API.
        authoritative_name -> name.name
        begin_date -> name.begin
        end_date -> name.end

    The identifier field is the absolute url to the name detail
    page for the model instance.
    """
    authoritative_name = serializers.CharField(source='name')
    begin_date = serializers.CharField(source='begin')
    name_type = serializers.SerializerMethodField()
    end_date = serializers.CharField(source='end')
    links = IdentifierSerializer(many=True, source='identifier_set')
    notes = NoteSerializer(many=True, source='note_set')
    variants = VariantSerializer(many=True, source='variant_set')
    identifier = serializers.HyperlinkedIdentityField(
        view_name='name:detail', lookup_field='name_id')

    class Meta:
        model = models.Name
        fields = ('authoritative_name', 'name_type', 'begin_date', 'end_date',
                  'identifier', 'links', 'notes', 'variants',)

    def get_name_type(self, obj):
        """Sets the name_type field.

        Returns the Name Type label, instead of the Name Type ID, which
        is the default behavior.
        """
        return obj.get_name_type_label().lower()


class NameSearchSerializer(serializers.ModelSerializer):
    """Name Model Serializer for the Name search/autocompletion
    endpoint.

    The following fields have been renamed for backwards compatibility
    with previous versions of the API.
        begin_date -> name.begin
        type -> name.get_name_type_label()
        label -> Formats name.name and name.disambiguation.

    The URL field is the absolute url to the name detail page for
    the model instance.
    """
    begin_date = serializers.CharField(source='begin')
    type = serializers.SerializerMethodField()
    label = serializers.SerializerMethodField()
    URL = serializers.HyperlinkedIdentityField(
        view_name='name:detail', lookup_field='name_id')

    class Meta:
        model = models.Name
        fields = ('id', 'name', 'label', 'type', 'begin_date',
                  'disambiguation', 'URL')

    def get_type(self, obj):
        """Sets the type field.

        Returns the Name Type label, instead of the Name Type ID, which
        is the default behavior.
        """
        return obj.get_name_type_label().lower()

    def get_label(self, obj):
        """Sets the label field.

        Returns a string in the form of
            "<name.name> (<name.disambiguation>)"
        """
        if obj.disambiguation:
            return u'{0} ({1})'.format(obj.name, obj.disambiguation)
        return obj.name


class LocationSerializer(serializers.ModelSerializer):
    """Serailizer for the Locations Model.

    This includes the related Name via the belong_to_name field. The
    belong_to_name field uses the NameSerializer to nest the related
    Name model.
    """
    belong_to_name = NameSerializer()

    class Meta:
        model = models.Location


class NameStatisticsMonthSerializer(serializers.Serializer):
    """Serializer for the NameStatisticsMonth object."""
    total = serializers.IntegerField()
    total_to_date = serializers.IntegerField()
    month = serializers.DateTimeField()


class NameStatisticsTypeSerializer(serializers.Serializer):
    """Serializer for the NameStatisticsType object.

    This serializer utilizes the NameStatisticsTypeMonth to serialize
    the NameStatisticsMonth instances that the object instance contains.
    """
    running_total = serializers.IntegerField()
    stats = NameStatisticsMonthSerializer(many=True)


class NameStatisticsSerializer(serializers.Serializer):
    """Serializer for the NameStatistics object.

    This serializer utilizes the NameStatisticsTypeSerializer to
    serialize the NameStatisticsType instances that the object instance
    contains.
    """
    created = NameStatisticsTypeSerializer()
    modified = NameStatisticsTypeSerializer()
    name_type_totals = serializers.DictField()



from django import http
from django.shortcuts import get_object_or_404

from rest_framework.renderers import JSONRenderer
from . import serializers, stats as statistics
from ..decorators import jsonp
from ..models import Name, Location
from ..utils import filter_names


class JSONResponse(http.HttpResponse):
    """HTTP Response object for returning JSON data."""

    def __init__(self, data, **kwargs):
        content = JSONRenderer().render(data, renderer_context={'indent': 4})
        kwargs['content_type'] = 'application/json'
        super(JSONResponse, self).__init__(content, **kwargs)


def name_json(request, name_id):
    """Returns result of name query in json format."""
    name = get_object_or_404(Name, name_id=name_id)
    data = serializers.NameSerializer(name, context={'request': request})

    return JSONResponse(data.data)


def stats_json(request):
    stats = statistics.NameStatistics()
    data = serializers.NameStatisticsSerializer(stats)
    return JSONResponse(data.data)


@jsonp
def search_json(request):
    """Gets the JSON encoded version of the Names matching
    the query parameters.

    This also provides the endpoint used for autocompletion
    during search.
    """
    names = filter_names(request)

    # If the request is AJAX, then it is most likely for autocompletion,
    # so we will only return first 10 names.
    if request.is_ajax():
        names = names[:10]

    data = serializers.NameSearchSerializer(
        names, many=True, context={'request': request})

    response = JSONResponse(data.data)
    response['Access-Control-Allow-Origin'] = '*'
    response['Access-Control-Allow-Headers'] = 'X-Requested-With'

    return response


def locations_json(request):
    """Presents the Locations and related Names serialized into JSON."""
    if request.is_ajax():
        locations = Location.objects.filter(status=0)

        data = serializers.LocationSerializer(
            locations, many=True, context={'request': request})

        return JSONResponse(data.data)
    return http.HttpResponseNotFound()



from django.conf.urls import url
from django.contrib import admin

from . import views, feeds
from .api import views as api


admin.autodiscover()

urlpatterns = [
    url(r'^$', views.landing, name='landing'),
    url(r'about/$', views.about, name='about'),
    url(r'export/$', views.export, name='export'),
    url(r'feed/$', feeds.NameAtomFeed(), name='feed'),
    url(r'label/(?P<name_value>.*)$', views.label, name='label'),
    url(r'locations.json/$', api.locations_json, name='locations-json'),
    url(r'map/$', views.locations, name='map'),
    url(r'opensearch.xml$', views.opensearch, name='opensearch'),
    url(r'search/$', views.SearchView.as_view(), name='search'),
    url(r'search.json$', api.search_json, name="search-json"),
    url(r'stats.json/$', api.stats_json, name='stats-json'),
    url(r'stats/$', views.stats, name='stats'),
    url(r'(?P<name_id>.*).json$', api.name_json, name='detail-json'),
    url(r'(?P<name_id>.*).mads.xml$', views.mads_serialize, name='mads-serialize'),
    url(r'(?P<name_id>[^/]+)/', views.detail, name='detail')
]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('name', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='location',
            name='status',
            field=models.IntegerField(default=0, choices=[(0, b'current'), (1, b'former')]),
        ),
        migrations.AlterField(
            model_name='name',
            name='name_type',
            field=models.IntegerField(choices=[(0, b'Personal'), (1, b'Organization'), (2, b'Event'), (3, b'Software'), (4, b'Building')]),
        ),
        migrations.AlterField(
            model_name='variant',
            name='variant_type',
            field=models.IntegerField(help_text=b'Choose variant type.', choices=[(0, b'Acronym'), (1, b'Abbreviation'), (2, b'Translation'), (3, b'Expansion'), (4, b'Other')]),
        ),
    ]



#! /usr/bin/env python
import os
import re
from setuptools import setup, find_packages

version = ''
with open('name/__init__.py', 'r') as fd:
    version = re.search(r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]',
                        fd.read(), re.MULTILINE).group(1)

readme = ''
with open(os.path.join(os.path.dirname(__file__), 'README.md')) as fd:
    readme = fd.read()

install_requires = [
    'python-dateutil',
    'markdown2',
    'djangorestframework >= 3.1,<3.3.0',
    'pynaco == 0.1.0'
]

setup(
    name='django-name',
    version=version,
    packages=find_packages(exclude=["tests.*", "tests"]),
    include_package_data=True,
    license='BSD',
    description='Name Authority App for Django.',
    long_description=readme,
    keywords='django name citation',
    author='University of North Texas Libraries',
    author_email='mark.phillips@unt.edu',
    url='https://github.com/unt-libraries/django-name',
    zip_safe=False,
    install_requires=install_requires,
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: End Users/Desktop',
        'Natural Language :: English',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Topic :: Internet :: WWW/HTTP :: WSGI :: Application'
    ]
)



from django.conf import settings

from . import __title__

# App level settings for the Name feed.
NAME_FEED_AUTHOR_NAME = getattr(settings, 'NAME_FEED_AUTHOR_NAME', __title__)

NAME_FEED_AUTHOR_EMAIL = getattr(settings, 'NAME_FEED_AUTHOR_EMAIL', None)

NAME_FEED_AUTHOR_LINK = getattr(settings, 'NAME_FEED_AUTHOR_LINK', None)

# App level settings for custom branding.
NAME_APP_TITLE = getattr(settings, 'NAME_APP_TITLE', __title__)

NAME_ADMIN_EMAIL = getattr(settings, 'NAME_ADMIN_EMAIL', None)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='BaseTicketing',
            fields=[
                ('id', models.AutoField(serialize=False, primary_key=True)),
                ('stub', models.BooleanField(default=True, unique=True)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Identifier',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('value', models.CharField(max_length=500)),
                ('visible', models.BooleanField(default=True)),
                ('order', models.IntegerField(default=0)),
            ],
            options={
                'ordering': ['order', 'type'],
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Identifier_Type',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('label', models.CharField(help_text=b'What kind of data is this? Personal website? Twitter?', max_length=255)),
                ('icon_path', models.CharField(help_text=b'Path to icon image?', max_length=255, blank=True)),
                ('homepage', models.URLField(help_text=b'Homepage of label. Twitter.com, Facebook.com, etc', blank=True)),
            ],
            options={
                'ordering': ['label'],
                'verbose_name': 'Identifier Type',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Location',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('latitude', models.DecimalField(help_text=b'\n    <strong>\n        <a target="_blank" href="http://itouchmap.com/latlong.html">\n            iTouchMap\n        </a>\n        : this service might be useful for filling in the lat/long data\n    </strong>\n    ', max_digits=13, decimal_places=10)),
                ('longitude', models.DecimalField(help_text=b'\n    <strong>\n        <a target="_blank" href="http://itouchmap.com/latlong.html">\n            iTouchMap\n        </a>\n        : this service might be useful for filling in the lat/long data\n    </strong>\n    ', max_digits=13, decimal_places=10)),
                ('status', models.IntegerField(default=0, max_length=2, choices=[(0, b'current'), (1, b'former')])),
            ],
            options={
                'ordering': ['status'],
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Name',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(help_text=b'Please use the general reverse order: LAST, FIRST', max_length=255)),
                ('normalized_name', models.CharField(help_text=b'NACO normalized form of the name', max_length=255, editable=False)),
                ('name_type', models.IntegerField(max_length=1, choices=[(0, b'Personal'), (1, b'Organization'), (2, b'Event'), (3, b'Software'), (4, b'Building')])),
                ('begin', models.CharField(help_text=b'Conforms to EDTF format YYYY-MM-DD', max_length=25, blank=True)),
                ('end', models.CharField(help_text=b'Conforms to EDTF format YYYY-MM-DD', max_length=25, blank=True)),
                ('disambiguation', models.CharField(help_text=b'Clarify to whom or what this record pertains.', max_length=255, blank=True)),
                ('biography', models.TextField(help_text=b'Compatible with MARKDOWN', blank=True)),
                ('record_status', models.IntegerField(default=0, choices=[(0, b'Active'), (1, b'Deleted'), (2, b'Suppressed')])),
                ('date_created', models.DateTimeField(auto_now_add=True)),
                ('last_modified', models.DateTimeField(auto_now=True)),
                ('name_id', models.CharField(unique=True, max_length=10, editable=False)),
                ('merged_with', models.ForeignKey(related_name=b'merged_with_name', blank=True, to='name.Name', null=True)),
            ],
            options={
                'ordering': ['name'],
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Note',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('note', models.TextField(help_text=b'Enter notes about this record here')),
                ('note_type', models.IntegerField(choices=[(0, b'Biographical/Historical'), (1, b'Deletion Information'), (2, b'Nonpublic'), (3, b'Source'), (4, b'Other')])),
                ('belong_to_name', models.ForeignKey(to='name.Name')),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Variant',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('variant_type', models.IntegerField(help_text=b'Choose variant type.', max_length=50, choices=[(0, b'Acronym'), (1, b'Abbreviation'), (2, b'Translation'), (3, b'Expansion'), (4, b'Other')])),
                ('variant', models.CharField(help_text=b'Fill in the other name variants, if any.', max_length=255)),
                ('normalized_variant', models.CharField(help_text=b'NACO normalized variant text', max_length=255, editable=False)),
                ('belong_to_name', models.ForeignKey(to='name.Name')),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.AlterUniqueTogether(
            name='name',
            unique_together=set([('name', 'name_id')]),
        ),
        migrations.AddField(
            model_name='location',
            name='belong_to_name',
            field=models.ForeignKey(to='name.Name'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='identifier',
            name='belong_to_name',
            field=models.ForeignKey(to='name.Name'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='identifier',
            name='type',
            field=models.ForeignKey(help_text=b"Catagorize this record's identifiers here", to='name.Identifier_Type'),
            preserve_default=True,
        ),
    ]



from django import template
from django.core.urlresolvers import reverse

register = template.Library()


@register.simple_tag(name="absolute_url", takes_context=True)
def absolute_url(context, name, *args):
    request = context.get('request')
    return request.build_absolute_uri(
        reverse(name, args=filter(None, list(args))))
